import org.scalajs.dom.{document, html}

object Editor {
  type Cursor = Int
  def empty = Editor("",0)
}

case class Editor(content: String, cursor: Editor.Cursor) {
  def applyOperation(operation: TextOperation.Operation): Editor = copy(TextOperation.applyOp(content,operation))

  def insert(c: Char) = {
    val operation = List.fill(cursor)(Retain) ++ (Insert(c) :: List.fill(content.length - cursor)(Retain))
    (Option(operation),Editor(TextOperation.applyOp(content,operation),cursor + 1))
  }

  def backspace = if (cursor > 0) {
    val operation = List.fill(cursor - 1)(Retain) ++ (Delete :: List.fill(content.length - cursor)(Retain))
    (Option(operation),Editor(TextOperation.applyOp(content,operation),cursor - 1))
  } else (None,this)

  def moveLeft =
    copy(cursor = Math.max(0,cursor - 1))

  def moveRight =
    copy(cursor = Math.min(content.length,cursor + 1))

  /**
    * Measure the size of a rendered string
    * @param text the string to render
    * @return width and height of the rendered text
    */
  private def measure(text: String): (Int,Int) = {
    val measure = document.getElementById("measure")
    val width = if (text.isEmpty || text.last == '\n') 0
    else {
      measure.textContent = text.lines.toStream.last
      measure.clientWidth
    }
    measure.textContent = if (text.isEmpty || text.last == '\n') text + " " else text
    (width,measure.clientHeight)
  }

  def render(elem: html.Div) = {
    val (w,h) = measure(content.take(this.cursor))
    val cursor = document.createElement("div").asInstanceOf[html.Div]
    cursor.style.left = w + "px"
    cursor.style.top = h + "px"
    cursor.classList.add("caret")
    elem.textContent = content
    elem.appendChild(cursor)
  }
}