import io.circe.{Decoder, DecodingFailure, Encoder, Json}
import io.circe.syntax._

sealed trait ClientMessage
case class ClientEdit(
  rev: Int,
  op: TextOperation.Operation) extends ClientMessage

sealed trait ServerMessage
case object Ack extends ServerMessage
case class RemoteEdit(
  op: TextOperation.Operation
) extends ServerMessage

object ClientMessage {
  implicit val encodeAction =
    Encoder.instance[TextAction] {
      case Retain => 1.asJson
      case Insert(c) => c.asJson
      case Delete => (-1).asJson
    }

  implicit val encodeClientMessage = Encoder.instance[ClientMessage] {
    case ClientEdit(rev,op) => Json.obj(
      "rev" -> rev.asJson,
      "op" -> op.asJson
    )
  }
}

object ServerMessage {
  implicit val parseAction =
    Decoder.decodeChar.map[TextAction](Insert) or
    Decoder.decodeInt.emap[TextAction] {
      case -1 => Right(Delete)
      case 1 => Right(Retain)
      case other => Left("invalid text action")
    }


  val parseAck = Decoder.decodeString.emap[ServerMessage] {
    case "ack" => Right(Ack)
    case other => Left("invalid server message")
  }

  val parseRemoteEdit = Decoder
    .decodeList[TextAction]
    .map[ServerMessage](RemoteEdit)

  implicit val parseServerMessage =
    parseAck or parseRemoteEdit
}